#if !defined(ENTETE)
#define ENTETE

/* include files */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <ctype.h>

/* several defines */
#define INIT     -1

#define VALID     1
#define INVALID   0

#define fC_x_fG   0
#define GpC_o     1

#define T        1
#define F        0

#define HOMO    1
#define MUS     2

/* scores and colors for html table */
#define SCOR1		0.25
#define SCOR2		0.5
#define SCOR3		0.75
#define SCOR4		1.1

#define COLOR1		"#EEC900"
#define COLOR2		"#EEADOE"
#define COLOR3		"EE00000"
#define COLOR4		"0000000"
#define BACKGCLOR	"#F0F0F0"


/******************************************************/
/*                      HUMAN DATA                    */
/******************************************************/
#define COEF_LG_HOMO 		0.0005927055
#define COEF_GC_HOMO		4.043293
#define COEF_CpG_HOMO 		4.83027
#define CONST_START_HOMO 	-7.271471

#define COEF_ATskew_HOMO 	-11.01590
#define COEF_GCskew_HOMO 	13.44387
#define CONST_SENS_HOMO 	0.02853

#define SEN_HOMO1 0.92
#define SEN_HOMO2 0.72
#define SEN_HOMO3 0.48
#define SEN_HOMO4 0.32
#define SEN_HOMO5 0.15
#define SEN_HOMO6 0.06
#define SEN_HOMO7 0.02
#define SEN_HOMO8 0.01
#define SEN_HOMO9 -1.00

#define SPE_HOMO1 0.48
#define SPE_HOMO2 0.51
#define SPE_HOMO3 0.59
#define SPE_HOMO4 0.70
#define SPE_HOMO5 0.73
#define SPE_HOMO6 0.69
#define SPE_HOMO7 0.80
#define SPE_HOMO8 1.00
#define SPE_HOMO9 -1.00


/******************************************************/
/*                      MOUSE DATA                    */
/******************************************************/
#define COEF_LG_MUS 		-0.00008749142
#define COEF_GC_MUS		15.27366
#define COEF_CpG_MUS 		16.38997
#define CONST_START_MUS 	-19.4423

#define COEF_ATskew_MUS 	-12.3270
#define COEF_GCskew_MUS 	8.8730
#define CONST_SENS_MUS 		0.2161

#define SEN_MUS1 1.00
#define SEN_MUS2 0.96
#define SEN_MUS3 0.93
#define SEN_MUS4 0.82
#define SEN_MUS5 0.72
#define SEN_MUS6 0.67
#define SEN_MUS7 0.60
#define SEN_MUS8 0.45
#define SEN_MUS9 0.26

#define SPE_MUS1 0.58
#define SPE_MUS2 0.67
#define SPE_MUS3 0.74
#define SPE_MUS4 0.76
#define SPE_MUS5 0.77
#define SPE_MUS6 0.82
#define SPE_MUS7 0.82
#define SPE_MUS8 0.88
#define SPE_MUS9 0.92

/* Path to pictures */
#define PATHPICTURES "http://pbil.univ-lyon1.fr/binaries/"
#define PATHHOME "file:/mnt/users/ponger/CpGProD/html/"


typedef int FLAG;
typedef int VALIDATION;


/* To store the characteristics of the sequences */
struct sequence
	{
	char * name;
	char *seq;
	int lg;
	double gc;
	double cpgoe;
	int cgi_nb;
	};
	
/* To store the characteritics of the CGIs */	
struct CGI
	{
	int begin;
	int end;
	int lg;
	double gc;
	double cpgoe;
	double cpg_gpc;
	double ATskew;
	double GCskew;
	double score_start;
	double proba_start;
	double score_strand;
	double proba_strand;
	};
	
/* To store the parameters used to search the CGIs */	
struct parameters
	{	
	int w_lg;		/*window's length*/
	int w_st;		/*window's step*/
	double mn_w_gc;		/*minimal G+C frequency of window*/
	double mn_w_cpg;	/*(G+C) frequency _independant_ CpGo/e minimal ratio of window*/
	double mn_w_cpg_fgc;	/*(G+C) frequency _dependant_ CpGo/e minimal ratio of window*/

	int mn_i_lg;		/*minimal length of islands*/
	double mn_i_gc;		/*minimal (G+C) frequency of islands*/
	double mn_i_cpg;	/*(G+C) frequency _independant_ CpGo/e minimal ratio of islands*/
	double mn_i_cpg_fgc;	/*(G+C) frequency _dependant_ CpGo/e minimal ratio of islands*/

	int mx_lg_between_i;	/*maximal length to group islands*/

	FLAG test_i;		/*VALID = test island before add a new window*/
	FLAG gp_n_i;            /*VALID = group neighboring islands*/
	FLAG CpG_e;		/*method to compute CpG expected*/
	double mn_b;		/*minimal frequency of known nucleotide in the analyzed sequence*/
	FLAG samescale;	
	};
	
/* To store the coeficients to compute the start score */
struct startScoreCoef
	{
	double COEF_LG;
	double COEF_GC;
	double COEF_CpG;
	double CONST_START;
	};
	
/* To store the coeficients to compute the strand score */
struct strandScoreCoef
	{
	double COEF_ATskew;
	double COEF_GCskew;
	double CONST_SENS;
	};

/* To store sensivity and specificity of species */
struct sensANDspec
	{
	double sensivity[9];
	double specificity[9];
	};

/* To read squences at FASTA format */
int read_fasta    (FILE *, struct sequence **);


/* To write CGI information */
void write_table_header(FILE *);
int write_CGIs(char *, struct CGI *, int, FILE *);
int write_CGIs_table_predict(char *, struct CGI *, int, FILE *);
void write_table_tail(FILE *);

void write_table_header_html(char * , FILE *);
int write_CGIs_html(char *, struct CGI *, int, FILE *);
int write_CGIs_table_predict_html(char *, struct CGI *, int, FILE *);
void write_table_tail_html(FILE *);

/* To write output page */
void write_tail(FILE *);
void write_header(FILE *);

void write_tail_html(FILE *);
void write_header_html(FILE *, int);

/* To search CGIs */
int search_CGIs(char *, struct CGI **, struct parameters);
int test_window(int, int, int, int, int, struct parameters);
int test_island(int, int, int, int , int , int , struct parameters );
int move_limit(char *, int, int, int*, int);
int test_CGI(int, int, int, int, int, int, struct parameters);
int add_CGI(struct CGI **, int *, int, int, int, int, int, int, int, int, int);
void move_CGI(struct CGI **, int, int);
int group_CGIs(char *, struct CGI **, int, struct parameters);
int compute_CpG_GpC_G_C_nb(char *, int, int, int *, int *, int *, int *, int *, int *, int *);

/* To alloc memory */
void *my_realloc(void *, size_t, char *);


/* To read parameters */
int init_parameters_CpGProD(struct parameters *, char*, char *, int *, int *, int, char **, int *, struct startScoreCoef *, struct strandScoreCoef *, struct sensANDspec *, char *, int *);

/* To write parameter information */
void write_parameters(char *, char *, struct parameters, struct startScoreCoef , struct strandScoreCoef, int, struct sensANDspec);

void write_parameters_html(struct parameters, struct startScoreCoef, struct strandScoreCoef, int, FILE *);

/* To write sequence information */
void write_sequences_info(struct sequence *, int);

void write_sequences_info_html(struct sequence *, int, FILE *);

/* To write errors */
void write_error_parameters(void);
void write_error_sequences(void);

void write_error_parameters_html(FILE *);
void write_error_sequences_html(FILE *);

/* To initialize the name of the FTP output file */
void init_outputfile_name_html(char *);

/* To write the message of correct syntaxe */
void write_message_syntaxe();

/* To compute start and strand scores */
void predict_promoter(struct CGI *, int, struct startScoreCoef);
void predict_strand(struct CGI *, int, struct strandScoreCoef);


/* To write picture of CGIs */
int write_CGI_picture(struct sequence*, struct CGI **, int, int, FILE *);



/* To open a file */
FILE* open_file(char *, int );

#endif

